@@ -1,10 +1,10 @@
 ##----------------------------------------------------------------------------
 ## Stripe API - ~/lib/Net/API/Stripe/List.pm
-## Version 0.2
-## Copyright(c) 2019 DEGUEST Pte. Ltd.
+## Version 0.1
+## Copyright(c) 2020 DEGUEST Pte. Ltd.
 ## Author: Jacques Deguest <jack@deguest.jp>
 ## Created 2019/11/02
-## Modified 2020/02/16
+## Modified 2020/03/29
 ## All rights reserved
 ## 
 ## This program is free software; you can redistribute  it  and/or  modify  it
@@ -18,9 +18,28 @@ BEGIN
     use parent qw( Net::API::Stripe::Generic );
 	# use B;
 	# use B::Deparse;
-    our( $VERSION ) = '0.2';
+    our( $VERSION ) = '0.2.2';
 };
 
+## Provide our own version of as_hash to avoid our helper methods from being called by Module::Generic::as_hash
+sub as_hash
+{
+	my $self = shift( @_ );
+	my $p = {};
+	$p = shift( @_ ) if( scalar( @_ ) == 1 && ref( $_[0] ) eq 'HASH' );
+	my $data = $self->_data;
+	my $res = [];
+	foreach my $this ( @$data )
+	{
+		if( $self->_is_object( $this ) && $this->can( 'as_hash' ) )
+		{
+			my $v = $this->as_hash( $p );
+			push( @$res, $v );
+		}
+	}
+	return( { data => $res } );
+}
+
 sub object { CORE::shift->_set_get_scalar( 'object', @_ ); }
 
 sub count { return( CORE::shift->_set_get_scalar( 'total_count', @_ ) ); }
@@ -31,7 +50,6 @@ sub has_more { CORE::shift->_set_get_boolean( 'has_more', @_ ); }
 
 sub url { CORE::shift->_set_get_uri( 'url', @_ ); }
 
-## sub total_count { CORE::shift->_set_get_scalar( 'total_count', @_ ); }
 sub total_count
 {
 	my $self = shift( @_ );
@@ -40,17 +58,8 @@ sub total_count
 		$self->_set_get_scalar( 'total_count', @_ );
 	}
 	my $total = $self->_set_get_scalar( 'total_count' );
-	# my( $p, $f, $l ) = caller;
-	# my $sub = ( caller( 1 ) )[3];
-	# $self->message( 3, "Called from package $p in file $f at line $l from sub $sub." );
 	if( !CORE::length( $total ) )
 	{
-		# my $code_ref = $self->can( 'data' );
-		# my $meta = B::svref_2object( $code_ref );
-		# my $deparse = B::Deparse->new( '-p', '-sC' );
-		# my $meth_body = $deparse->coderef2text( $code_ref );
-		# $self->message( 3, "Log handler code routine name is " . $meta->GV->NAME . " called in file " . $meta->GV->FILE . " at line " . $meta->GV->LINE, " with code: ", $meth_body );
-		# $self->message( 3, "Data is '", $self->data(), "' and data value is '", $self->{data}, "'." );
 		return( $self->_data->size );
 	}
 	else
@@ -96,16 +105,11 @@ sub next
 		limit => ( $self->{_limit} || 10 ),
 		};
 		my $hash = $self->parent->get( $self->url, $opts ) || return;
-# 		return( $self->error( "Cannot find the data property in the hash returned from the follow-on query to Stripe." ) ) if( !CORE::exists( $hash->{data} ) );
-# 		return( $self->error( "Data property is not an array reference." ) ) if( ref( $hash->{data} ) ne 'ARRAY' );
-# 		return( '' ) if( !scalar( @{$hash->{data}} ) );
-# 		return( $self->error( "I was expecting an array of hash reference, but instead I got: ", sub{ $self->dumper( $hahs->{data} ) } ) ) if( ref( $hash->{data}->[0] ) ne 'HASH' );
 		return( $self->error( "Cannot find property 'object' in this hash reference: ", sub{ $self->dumper( $hash ) } ) ) if( !CORE::exists( $hash->{object} ) );
 		my $class = $self->_object_type_to_class( $hash->{object} ) || return;
 		my $list = $self->parent->_response_to_object( $class, $hash ) || return;
 		$data = $list->_data;
-		## $self->{data} = $data;
-		$self->_data( $data );
+		$self->{data} = $data;
 		$self->{_pos} = 0;
 		return( '' ) if( !scalar( @$data ) );
 		return( $data->[ $self->{_pos} ] );
@@ -142,8 +146,8 @@ sub prev
 		my $class = $self->_object_type_to_class( $hash->{object} ) || return;
 		my $list = $self->parent->_response_to_object( $class, $hash ) || return;
 		$data = $list->_data;
-		## $self->{data} = $data;
-		$self->_data( $data );
+		$self->{data} = $data;
+		## $self->_data( $data );
 		$self->{_pos} = $#$data;
 		return( '' ) if( !scalar( @$data ) );
 		return( $data->[ $self->{_pos} ] );
@@ -207,47 +211,46 @@ sub _check
 sub _data
 {
 	my $self = CORE::shift( @_ );
-	# my $field = shift( @_ );
-	# my( $pack, $file, $line ) = caller;
-	# require Devel::StackTrace;
-	# my $trace = Devel::StackTrace->new;
-	# $self->message( 3, "data called from package $pack in file $file at line $line with arguments: '", join( "', '", @_ ), "'." );
-	## $self->message( 3, "data called with arguments: '", join( "', '", @_ ), "'. Stack trace: ", $trace->as_string );
 	my $field = 'data';
 	if( @_ )
 	{
 		my $ref = CORE::shift( @_ );
-		# $self->message( 3, "Received '$ref' (", ref( $ref ), ")." );
-    	return( $self->error( "I was expecting an array ref, but instead got '$ref'" ) ) if( !$self->_is_array( $ref ) );
+    	return( $self->error( "I was expecting an array ref, but instead got '$ref'. _is_array returned: '", $self->_is_array( $ref ), "'" ) ) if( !$self->_is_array( $ref ) );
     	my $arr = [];
-    	# $self->messagef( 3, "Array of %d items provided.", scalar( @$ref ) );
-    	if( scalar( @$ref ) )
+    	## Are we provided with an array of existing objects? No need to do anything then
+    	if( $self->_is_object( $ref->[0] ) )
     	{
-    		return( $self->error( "I was expecting an array of hash reference, but instead of hash I found $ref->[0]" ) ) if( ref( $ref->[0] ) ne 'HASH' );
-    		return( $self->error( "Found an hash reference in this array, but it is empty: ", sub{ $self->dumper( $ref ) } ) ) if( !scalar( keys( %{$ref->[0]} ) ) );
-			my $type = $ref->[0]->{object} || return( $self->error( "I was expecting a string in property 'object', but found nothing: ", sub{ $self->dumper( $ref ) } ) );
-			my $class = $self->_object_type_to_class( $type ) || return( $self->error( "Could not find corresponding class for ojbect type \"$type\"." ) );
-			$arr = $self->_set_get_object_array_object( $field, $class, $ref );
-			# $self->messagef( 3, "_set_get_object_array returned %d items ($arr).", scalar( @$arr ) );
-			## Store this value used by next() and prev() to replicate the query with the right limit
-			## If initial query made by the user was 10 this array would be 10 or less if there is no more data
+    		$arr = $ref;
+    	}
+    	else
+    	{
+			if( scalar( @$ref ) )
+			{
+				my $type;
+				if( $self->_is_object( $ref->[0] ) )
+				{
+					return( $self->error( "I found an array of objects, but they do not have the method \"object\"." ) ) if( !$ref->[0]->can( 'object' ) );
+					$type = $ref->[0]->object || return( $self->error( "Somehow, the object property for this object (", $ref->[0], ") is empty." ) );
+				}
+				else
+				{
+					return( $self->error( "I was expecting an array of hash reference, but instead of hash I found $ref->[0]" ) ) if( ref( $ref->[0] ) ne 'HASH' );
+					return( $self->error( "Found an hash reference in this array, but it is empty: ", sub{ $self->dumper( $ref ) } ) ) if( !scalar( keys( %{$ref->[0]} ) ) );
+					$type = $ref->[0]->{object} || return( $self->error( "I was expecting a string in property 'object', but found nothing: ", sub{ $self->dumper( $ref ) } ) );
+				}
+				my $class = $self->_object_type_to_class( $type ) || return( $self->error( "Could not find corresponding class for ojbect type \"$type\"." ) );
+				$arr = $self->_set_get_object_array_object( $field, $class, $ref );
+				## Store this value used by next() and prev() to replicate the query with the right limit
+				## If initial query made by the user was 10 this array would be 10 or less if there is no more data
+			}
     	}
-    	#else
-    	#{
-    	#	$self->message( 3, "Array $ref is empty." );
-    	#}
     	$self->{ $field } = $arr;
 	}
 	if( !$self->{ $field } || !$self->_is_object( $self->{ $field } ) )
 	{
-		# $self->message( 3, "Currently value for field '$field' is \"", $self->{ $field }, "\", which is either empty or not an object." );
         my $o = Module::Generic::Array->new( $self->{ $field } );
 		$self->{ $field } = $o;
-		# $self->message( 3, "Value for field \"$field\" is now: \"", $self->{ $field }, "\"." );
 	}
-	# my( $p, $f, $l ) = caller;
-	# my $sub = ( caller( 1 ) )[3];
-	# $self->message( 3, "Called from package $p in file $f at line $l from sub $sub and returning ", $self->{ $field } );
 	return( $self->{ $field } );
 }
 
@@ -263,9 +266,17 @@ Net::API::Stripe::List - Stripe List Object
 
 =head1 SYNOPSIS
 
+    my $stripe = Net::API::Stripe->new( conf_file => 'settings.json' ) || die( Net::API::Stripe->error );
+    my $list = $stripe->customers( 'list' ) || die( $stripe->error );
+    printf( "%d total customer(s) found\n", $list->count );
+    while( my $cust = $list->next )
+    {
+        printf( "Customer %s with e-mail has a balance of %s\n", $cust->name, $cust->email, $cust->balance->format_money( 0, '¥' ) );
+    }
+
 =head1 VERSION
 
-    0.2
+    0.2.2
 
 =head1 DESCRIPTION
 
@@ -273,31 +284,31 @@ This is a package with a set of useful methods to be inherited by various Stripe
 
 =over 4
 
-=item C<Net::API::Stripe::Billing::Invoice::Lines>
+=item L<Net::API::Stripe::Billing::Invoice::Lines>
 
-=item C<Net::API::Stripe::Billing::Subscription::Items>
+=item L<Net::API::Stripe::Billing::Subscription::Items>
 
-=item C<Net::API::Stripe::Charge::Refunds>
+=item L<Net::API::Stripe::Charge::Refunds>
 
-=item C<Net::API::Stripe::Connect::Account::ExternalAccounts>
+=item L<Net::API::Stripe::Connect::Account::ExternalAccounts>
 
-=item C<Net::API::Stripe::Connect::ApplicationFee::Refunds>
+=item L<Net::API::Stripe::Connect::ApplicationFee::Refunds>
 
-=item C<Net::API::Stripe::Connect::Transfer::Reversals>
+=item L<Net::API::Stripe::Connect::Transfer::Reversals>
 
-=item C<Net::API::Stripe::Customer::Sources>
+=item L<Net::API::Stripe::Customer::Sources>
 
-=item C<Net::API::Stripe::Customer::Subscription>
+=item L<Net::API::Stripe::Customer::Subscription>
 
-=item C<Net::API::Stripe::Customer::TaxIds>
+=item L<Net::API::Stripe::Customer::TaxIds>
 
-=item C<Net::API::Stripe::File::Links>
+=item L<Net::API::Stripe::File::Links>
 
-=item C<Net::API::Stripe::Order::Return>
+=item L<Net::API::Stripe::Order::Return>
 
-=item C<Net::API::Stripe::Payment::Intent::Charges>
+=item L<Net::API::Stripe::Payment::Intent::Charges>
 
-=item C<Net::API::Stripe::Sigma::ScheduledQueryRun::File::Links>
+=item L<Net::API::Stripe::Sigma::ScheduledQueryRun::File::Links>
 
 =back
 
@@ -307,19 +318,9 @@ This is a package with a set of useful methods to be inherited by various Stripe
 
 =item B<new>( %ARG )
 
-Creates a new C<Net::API::Stripe> objects.
+Creates a new L<Net::API::Stripe::List> object.
 It may also take an hash like arguments, that also are method of the same name.
 
-=over 8
-
-=item I<verbose>
-
-Toggles verbose mode on/off
-
-=item I<debug>
-
-Toggles debug mode on/off
-
 =back
 
 =head1 METHODS
@@ -372,11 +373,11 @@ Moves to the previous entry in the array
 	  "has_more": false,
 	  "data": [
 		{
-		  "id": "re_1DQFAzCeyNCl6fY2Ntw9eath",
+		  "id": "re_fake123456789",
 		  "object": "refund",
 		  "amount": 30200,
-		  "balance_transaction": "txn_1DQFAzCeyNCl6fY2VBP8MxtZ",
-		  "charge": "ch_1DQFAHCeyNCl6fY2ugNKel6w",
+		  "balance_transaction": "txn_fake123456789",
+		  "charge": "ch_fake123456789",
 		  "created": 1540736617,
 		  "currency": "jpy",
 		  "metadata": {},
@@ -409,7 +410,7 @@ L<https://stripe.com/docs/api>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright (c) 2018-2019 DEGUEST Pte. Ltd.
+Copyright (c) 2020-2020 DEGUEST Pte. Ltd.
 
 You can use, copy, modify and redistribute this package and associated
 files under the same terms as Perl itself.
